from __future__ import absolute_import
from __future__ import print_function
import os
import sys

import pyverilog.vparser.ast as vast
from pyverilog.ast_code_generator.codegen import ASTCodeGenerator

expected = """\

module top #
(
  parameter DATAWID = 32
)
(
  input CLK,
  input RST,
  output [7:0] led
);

  reg [DATAWID-1:0] count;
  assign led = count[DATAWID-1:DATAWID-8];

  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
    end else begin
      count <= (count + 1) * 2 + 1;
    end
  end


endmodule
"""

def test():
    datawid = vast.Parameter( 'DATAWID', vast.Rvalue(vast.IntConst('32')) )
    params = vast.Paramlist( [datawid] )
    clk = vast.Ioport( vast.Input('CLK') )
    rst = vast.Ioport( vast.Input('RST') )
    width = vast.Width( vast.IntConst('7'), vast.IntConst('0') )
    led = vast.Ioport( vast.Output('led', width=width) )
    ports = vast.Portlist( [clk, rst, led] )

    width = vast.Width( vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('1')), vast.IntConst('0') )
    count = vast.Reg('count', width=width)

    assign = vast.Assign(
        vast.Lvalue(vast.Identifier('led')), 
        vast.Rvalue(
            vast.Partselect(
                vast.Identifier('count'), # count
                vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('1')), # [DATAWID-1:
                vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('8'))))) # :DATAWID-8]

    sens = vast.Sens(vast.Identifier('CLK'), type='posedge')
    senslist = vast.SensList([ sens ])

    assign_count_true = vast.NonblockingSubstitution(
        vast.Lvalue(vast.Identifier('count')),
        vast.Rvalue(vast.IntConst('0')))
    if0_true = vast.Block([ assign_count_true ])

    # (count + 1) * 2
    count_plus_1 = vast.Plus(vast.Identifier('count'), vast.IntConst('1'))
    cp1_times_2 = vast.Times(count_plus_1, vast.IntConst('2'))
    cp1t2_plus_1 = vast.Plus(cp1_times_2, vast.IntConst('1'))
    assign_count_false = vast.NonblockingSubstitution(
        vast.Lvalue(vast.Identifier('count')),
        vast.Rvalue(cp1t2_plus_1))
    if0_false = vast.Block([ assign_count_false ])

    if0 = vast.IfStatement(vast.Identifier('RST'), if0_true, if0_false)
    statement = vast.Block([ if0 ])

    always = vast.Always(senslist, statement)

    items = []
    items.append(count)
    items.append(assign)
    items.append(always)

    ast = vast.ModuleDef("top", params, ports, items)
    
    codegen = ASTCodeGenerator()
    rslt = codegen.visit(ast)
    print(rslt)
    
    assert(expected == rslt)

if __name__ == '__main__':
    test()



#-------------------------------------------------------------------------------
# identifiervisitor.py
# 
# Identifier list generator in a nested operator 
#
# Copyright (C) 2015, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------
from __future__ import absolute_import
from __future__ import print_function
import sys
import os

from pyverilog.dataflow.visit import NodeVisitor

def getIdentifiers(node):
    v = IdentifierVisitor()
    v.visit(node)
    ids = v.getIdentifiers()
    return ids
    
class IdentifierVisitor(NodeVisitor):
    def __init__(self):
        self.identifiers = []

    def getIdentifiers(self):
        return tuple(self.identifiers)

    def reset(self):
        self.identifiers = []
    
    def visit_Identifier(self, node):
        self.identifiers.append(node.name)



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
"""

def test():
    filelist = [codedir + 'reset.v']
    topmodule = 'TOP'
    noreorder = False
    nobind = False
    include = None
    define = None

    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    #rslt = ''
    #print(rslt)
    #assert(expected == rslt)
    
if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
Directive:
Instance:
(led, 'led')
Term:
(Term name:led.CLK type:['Input'] msb:(IntConst 0) lsb:(IntConst 0))
(Term name:led.LED type:['Output', 'Reg'] msb:(IntConst 7) lsb:(IntConst 0))
(Term name:led.RST type:['Input'] msb:(IntConst 0) lsb:(IntConst 0))
(Term name:led.STEP type:['Parameter'] msb:(IntConst 31) lsb:(IntConst 0))
(Term name:led.count type:['Reg'] msb:(IntConst 31) lsb:(IntConst 0))
Bind:
(Bind dest:led.LED \
tree:(Branch Cond:(Terminal led.RST) \
True:(IntConst 0) \
False:(Branch Cond:(Operator Eq Next:(Terminal led.count),(Operator Minus Next:(Terminal led.STEP),(IntConst 1))) True:(Operator Plus Next:(Terminal led.LED),(IntConst 1)))))
(Bind dest:led.STEP tree:(IntConst 10))
(Bind dest:led.count \
tree:(Branch Cond:(Terminal led.RST) \
True:(IntConst 0) \
False:(Branch Cond:(Operator Eq Next:(Terminal led.count),(Operator Minus Next:(Terminal led.STEP),(IntConst 1))) True:(IntConst 0) False:(Operator Plus Next:(Terminal led.count),(IntConst 1)))))
"""

def test():
    filelist = [codedir + 'led.v']
    topmodule = 'led'
    noreorder = False
    nobind = False
    include = None
    define = None
    
    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    directives = analyzer.get_directives()
    instances = analyzer.getInstances()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    output = []
    
    output.append('Directive:\n')
    for dr in sorted(directives, key=lambda x:str(x)):
        output.append(str(dr))
        output.append('\n')

    output.append('Instance:\n')
    for module, instname in sorted(instances, key=lambda x:str(x[1])):
        output.append(str((module, instname)))
        output.append('\n')

    output.append('Term:\n')
    for tk, tv in sorted(terms.items(), key=lambda x:str(x[0])):
        output.append(tv.tostr())
        output.append('\n')
        
    output.append('Bind:\n')
    for bk, bv in sorted(binddict.items(), key=lambda x:str(x[0])):
        for bvi in bv:
            output.append(bvi.tostr())
            output.append('\n')
            
    rslt = ''.join(output)

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
from optparse import OptionParser

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.controlflow.splitter as splitter
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
from pyverilog.controlflow.active_analyzer import VerilogActiveConditionAnalyzer

def main():
    INFO = "Active condition analyzer"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python example_active_analyzer.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    canalyzer = VerilogActiveConditionAnalyzer(options.topmodule, terms, binddict, 
                                               resolved_terms, resolved_binddict, constlist)

    for target in options.searchtarget:
        signal = util.toTermname(target)

        active_conditions = canalyzer.getActiveConditions( signal )
        #active_conditions = canalyzer.getActiveConditions( signal, condition=splitter.active_modify )
        #active_conditions = canalyzer.getActiveConditions( signal, condition=splitter.active_unmodify )

        print('Active Cases: %s' % signal)
        for fsm_sig, active_conditions in sorted(active_conditions.items(), key=lambda x:str(x[0])):
            print('FSM: %s' % fsm_sig)
            for state, active_condition in sorted(active_conditions, key=lambda x:str(x[0])):
                s = []
                s.append('state: %d -> ' % state)
                if active_condition: s.append(active_condition.tocode())
                else: s.append('empty')
                print(''.join(s))

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.vparser.parser import VerilogCodeParser

try:
    from StringIO import StringIO
except:
    from io import StringIO

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
Source:  (at 2)
  Description:  (at 2)
    ModuleDef: led (at 2)
      Paramlist:  (at 2)
        Decl:  (at 4)
          Parameter: STEP, False (at 4)
            Rvalue:  (at 4)
              IntConst: 10 (at 4)
      Portlist:  (at 6)
        Ioport:  (at 7)
          Input: CLK, False (at 7)
        Ioport:  (at 8)
          Input: RST, False (at 8)
        Ioport:  (at 9)
          Output: LED, False (at 9)
            Width:  (at 9)
              IntConst: 7 (at 9)
              IntConst: 0 (at 9)
          Reg: LED, False (at 9)
            Width:  (at 9)
              IntConst: 7 (at 9)
              IntConst: 0 (at 9)
      Decl:  (at 12)
        Reg: count, False (at 12)
          Width:  (at 12)
            IntConst: 31 (at 12)
            IntConst: 0 (at 12)
      Always:  (at 14)
        SensList:  (at 14)
          Sens: posedge (at 14)
            Identifier: CLK (at 14)
        Block: None (at 14)
          IfStatement:  (at 15)
            Identifier: RST (at 15)
            Block: None (at 15)
              NonblockingSubstitution:  (at 16)
                Lvalue:  (at 16)
                  Identifier: count (at 16)
                Rvalue:  (at 16)
                  IntConst: 0 (at 16)
              NonblockingSubstitution:  (at 17)
                Lvalue:  (at 17)
                  Identifier: LED (at 17)
                Rvalue:  (at 17)
                  IntConst: 0 (at 17)
            Block: None (at 18)
              IfStatement:  (at 19)
                Eq:  (at 19)
                  Identifier: count (at 19)
                  Minus:  (at 19)
                    Identifier: STEP (at 19)
                    IntConst: 1 (at 19)
                Block: None (at 19)
                  NonblockingSubstitution:  (at 20)
                    Lvalue:  (at 20)
                      Identifier: count (at 20)
                    Rvalue:  (at 20)
                      IntConst: 0 (at 20)
                  NonblockingSubstitution:  (at 21)
                    Lvalue:  (at 21)
                      Identifier: LED (at 21)
                    Rvalue:  (at 21)
                      Plus:  (at 21)
                        Identifier: LED (at 21)
                        IntConst: 1 (at 21)
                Block: None (at 22)
                  NonblockingSubstitution:  (at 23)
                    Lvalue:  (at 23)
                      Identifier: count (at 23)
                    Rvalue:  (at 23)
                      Plus:  (at 23)
                        Identifier: count (at 23)
                        IntConst: 1 (at 23)
    ModuleDef: main (at 29)
      Paramlist:  (at 0)
      Portlist:  (at 30)
        Ioport:  (at 31)
          Input: CLK, False (at 31)
        Ioport:  (at 32)
          Input: RST, False (at 32)
        Ioport:  (at 33)
          Output: LED, False (at 33)
            Width:  (at 33)
              IntConst: 7 (at 33)
              IntConst: 0 (at 33)
      InstanceList: led (at 36)
        ParamArg: STEP (at 38)
          IntConst: 100 (at 38)
        Instance: inst_led, led (at 36)
          ParamArg: STEP (at 38)
            IntConst: 100 (at 38)
          PortArg: CLK (at 42)
            Identifier: CLK (at 42)
          PortArg: RST (at 43)
            Identifier: RST (at 43)
          PortArg: LED (at 44)
            Identifier: LED (at 44)
Line 1 : `default_nettype none
"""

def test():
    filelist = [codedir + 'led_main.v']
    output = 'preprocess.out'
    include = [codedir]
    define = ['STEP=100']
    
    parser = VerilogCodeParser(filelist,
                               preprocess_include=include,
                               preprocess_define=define)
    ast = parser.parse()
    directives = parser.get_directives()

    output = StringIO()
    ast.show(buf=output)

    for lineno, directive in directives:
        output.write('Line %d : %s' % (lineno, directive))
    
    rslt = output.getvalue()

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys

import pyverilog.vparser.ast as vast
from pyverilog.ast_code_generator.codegen import ASTCodeGenerator

expected = """\

module top
(
  input CLK,
  input RST,
  output [7:0] led
);

  assign led = 8;

endmodule
"""

def test():
    params = vast.Paramlist( [] )
    clk = vast.Ioport( vast.Input('CLK') )
    rst = vast.Ioport( vast.Input('RST') )
    width = vast.Width( vast.IntConst('7'), vast.IntConst('0') )
    led = vast.Ioport( vast.Output('led', width=width) )
    ports = vast.Portlist( (clk, rst, led) )
    items = [ vast.Assign( vast.Lvalue(vast.Identifier('led')),
                           vast.Rvalue(vast.IntConst('8'))) ]
    ast = vast.ModuleDef("top", params, ports, items)
    
    codegen = ASTCodeGenerator()
    rslt = codegen.visit(ast)
    
    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
from optparse import OptionParser

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.utils.version
import pyverilog.utils.util as util
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
from pyverilog.controlflow.active_range import VerilogActiveAnalyzer

def main():
    INFO = "Active condition analyzer (Obsoluted)"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python example_active_range.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    aanalyzer = VerilogActiveAnalyzer(options.topmodule, terms, binddict, 
                                      resolved_terms, resolved_binddict, constlist)

    for target in options.searchtarget:
        signal = util.toTermname(target)

        print('Active Conditions: %s' % signal)
        active_conditions = aanalyzer.getActiveConditions( signal )
        print(sorted(active_conditions, key=lambda x:str(x)))

        print('Changed Conditions')
        changed_conditions = aanalyzer.getChangedConditions( signal )
        print(sorted(changed_conditions, key=lambda x:str(x)))
    
        print('Changed Condition Dict')
        changed_conditiondict = aanalyzer.getChangedConditionsWithAssignments( signal )
        print(sorted(changed_conditiondict.items(), key=lambda x:str(x[0])))
    
        print('Unchanged Conditions')
        unchanged_conditions = aanalyzer.getUnchangedConditions( signal )
        print(sorted(unchanged_conditions, key=lambda x:str(x)))

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
from optparse import OptionParser

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.utils.version
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
from pyverilog.dataflow.subset import VerilogSubset

def main():
    INFO = "Subset generator from Verilog dataflow definitions"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python example_subset.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    optparser.add_option("--clockname",dest="clockname",
                         default="CLK",help="Clock signal name")
    optparser.add_option("--resetname",dest="resetname",
                         default="RST_X",help="Reset signal name")
    optparser.add_option("--clockedge",dest="clockedge",
                         default="posedge",help="Clock signal edge")
    optparser.add_option("--resetedge",dest="resetedge",
                         default="negedge",help="Reset signal edge")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       noreorder=options.noreorder,
                                       nobind=options.nobind,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    subset = VerilogSubset(options.topmodule, terms, binddict, 
                           resolved_terms, resolved_binddict, constlist)

    subset.set_clock_info(options.clockname, options.clockedge)
    subset.set_reset_info(options.resetname, options.resetedge)

    sub_binds, sub_terms = subset.getBindSourceSubset(options.searchtarget)
    terms, parameter, assign, always_clockedge, always_combination = subset.getSubset(options.searchtarget)

    for k, v in terms.items():
        print(v.tocode())
    for k, v in parameter.items():
        print(v.tocode())
    for k, v in assign.items():
        for vv in v:
            print(vv.tocode())
    for k, v in always_clockedge.items():
        for vv in v:
            print(vv.tocode())
    for k, v in always_combination.items():
        for vv in v:
            print(vv.tocode())

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.vparser.parser import VerilogCodeParser

try:
    from StringIO import StringIO
except:
    from io import StringIO

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
Source:  (at 3)
  Description:  (at 3)
    ModuleDef: main (at 3)
      Paramlist:  (at 3)
        Decl:  (at 5)
          Parameter: STEP, False (at 5)
            Rvalue:  (at 5)
              IntConst: 10 (at 5)
      Portlist:  (at 7)
        Ioport:  (at 8)
          Input: CLK, False (at 8)
        Ioport:  (at 9)
          Input: RST, False (at 9)
        Ioport:  (at 10)
          Output: LED, False (at 10)
            Width:  (at 10)
              IntConst: 7 (at 10)
              IntConst: 0 (at 10)
          Reg: LED, False (at 10)
            Width:  (at 10)
              IntConst: 7 (at 10)
              IntConst: 0 (at 10)
      Decl:  (at 13)
        Localparam: DELAY, False (at 13)
          Rvalue:  (at 13)
            IntConst: 10 (at 13)
      Always:  (at 15)
        SensList:  (at 15)
          Sens: posedge (at 15)
            Identifier: CLK (at 15)
        Block: None (at 15)
          IfStatement:  (at 16)
            Identifier: RST (at 16)
            Block: None (at 16)
              NonblockingSubstitution:  (at 17)
                Lvalue:  (at 17)
                  Identifier: LED (at 17)
                Rvalue:  (at 17)
                  IntConst: 0 (at 17)
            Block: None (at 18)
              NonblockingSubstitution:  (at 19)
                Lvalue:  (at 19)
                  Identifier: LED (at 19)
                Rvalue:  (at 19)
                  Plus:  (at 19)
                    Identifier: LED (at 19)
                    IntConst: 1 (at 19)
                DelayStatement:  (at 19)
                  Identifier: DELAY (at 19)
Line 1 : `timescale 1ns / 1ps
"""

def test():
    filelist = [codedir + 'delay.v']
    output = 'preprocess.out'
    include = None
    define = None
    
    parser = VerilogCodeParser(filelist,
                               preprocess_include=include,
                               preprocess_define=define)
    ast = parser.parse()
    directives = parser.get_directives()

    output = StringIO()
    ast.show(buf=output)

    for lineno, directive in directives:
        output.write('Line %d : %s' % (lineno, directive))
    
    rslt = output.getvalue()

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
from optparse import OptionParser

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.utils.version
from pyverilog.vparser.preprocessor import preprocess
from pyverilog.vparser.lexer import dump_tokens

def main():
    INFO = "Verilog Lexer"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python example_lexer.py file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()

    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()
        
    text = preprocess(filelist, 
                      include=options.include,
                      define=options.define)

    dump = dump_tokens(text)
    
    print(dump)

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.vparser.preprocessor import VerilogPreprocessor

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
`default_nettype none
module led #
  (
   parameter STEP = 10
   )
  (
   input CLK,
   input RST,
   output reg [7:0] LED
   );
  
  reg [31:0] count;
  
  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
      LED <= 0;
    end else begin
      if(count == STEP - 1) begin
        count <= 0;
        LED <= LED + 1;
      end else begin
        count <= count + 1;
      end
    end
  end
endmodule

module main
(
 input CLK,
 input RST,
 output [7:0] LED
 );
  
  led #
    (
     .STEP(100)
     )
  inst_led
    (
     .CLK(CLK),
     .RST(RST),
     .LED(LED)
     );
  
endmodule
"""

def test():
    filelist = [codedir + 'led_main.v']
    output = 'preprocess.out'
    include = [codedir]
    define = ['STEP=100']
    
    pre = VerilogPreprocessor(filelist, output, include, define)
    pre.preprocess()
    rslt = open(output).read()
    os.remove(output)
    
    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.vparser.ast as vast
from pyverilog.ast_code_generator.codegen import ASTCodeGenerator

def main():
    datawid = vast.Parameter( 'DATAWID', vast.Rvalue(vast.IntConst('32')) )
    params = vast.Paramlist( [datawid] )
    clk = vast.Ioport( vast.Input('CLK') )
    rst = vast.Ioport( vast.Input('RST') )
    width = vast.Width( vast.IntConst('7'), vast.IntConst('0') )
    led = vast.Ioport( vast.Output('led', width=width) )
    ports = vast.Portlist( [clk, rst, led] )

    width = vast.Width( vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('1')), vast.IntConst('0') )
    count = vast.Reg('count', width=width)

    assign = vast.Assign(
        vast.Lvalue(vast.Identifier('led')), 
        vast.Rvalue(
            vast.Partselect(
                vast.Identifier('count'), # count
                vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('1')), # [DATAWID-1:
                vast.Minus(vast.Identifier('DATAWID'), vast.IntConst('8'))))) # :DATAWID-8]

    sens = vast.Sens(vast.Identifier('CLK'), type='posedge')
    senslist = vast.SensList([ sens ])

    assign_count_true = vast.NonblockingSubstitution(
        vast.Lvalue(vast.Identifier('count')),
        vast.Rvalue(vast.IntConst('0')))
    if0_true = vast.Block([ assign_count_true ])

    # count + 1
    count_plus_1 = vast.Plus(vast.Identifier('count'), vast.IntConst('1'))
    assign_count_false = vast.NonblockingSubstitution(
        vast.Lvalue(vast.Identifier('count')),
        vast.Rvalue(count_plus_1))
    if0_false = vast.Block([ assign_count_false ])

    if0 = vast.IfStatement(vast.Identifier('RST'), if0_true, if0_false)
    statement = vast.Block([ if0 ])

    always = vast.Always(senslist, statement)

    items = []
    items.append(count)
    items.append(assign)
    items.append(always)

    ast = vast.ModuleDef("top", params, ports, items)
    
    codegen = ASTCodeGenerator()
    rslt = codegen.visit(ast)
    print(rslt)

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import pyverilog.vparser.ast as vast
from pyverilog.utils.identifiervisitor import getIdentifiers

def main():
    a = vast.Identifier('a')
    b = vast.Identifier('b')
    c = vast.Plus(a, b)

    ids = getIdentifiers(c)
    print(ids)

if __name__ == '__main__':
    main()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
(Bind dest:TOP.out0 \
tree:(Operator Uand Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out1 \
tree:(Operator Unand Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out2 \
tree:(Operator Uor Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out3 \
tree:(Operator Unor Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out4 \
tree:(Operator Uxor Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out5 \
tree:(Operator Uxnor Next:(Concat Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in2) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in3) MSB:(IntConst 0) LSB:(IntConst 0)),(Partselect Var:(Terminal TOP.in4) MSB:(IntConst 0) LSB:(IntConst 0)))))
(Bind dest:TOP.out6 \
tree:(Operator Ulnot Next:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0))))
(Bind dest:TOP.out7 \
tree:(Partselect Var:(Terminal TOP.in1) MSB:(IntConst 0) LSB:(IntConst 0)))
"""

def test():
    filelist = [codedir + 'primitive.v']
    topmodule = 'TOP'
    noreorder = False
    nobind = False
    include = None
    define = None
    
    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    directives = analyzer.get_directives()
    instances = analyzer.getInstances()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    output = []
    
    for bk, bv in sorted(binddict.items(), key=lambda x:str(x[0])):
        for bvi in bv:
            output.append(bvi.tostr())
            output.append('\n')
            
    rslt = ''.join(output)

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()
